from .economy import *

__title__ = "DiscordEconomy"
__summary__ = "Discord.py, other libs, and forks(pycord, nextcord etc.) extension to create economy easily."
__uri__ = "https://github.com/Nohet/DiscordEconomy"
__version__ = "1.3.5"
__author__ = "Nohet"
__email__ = "igorczupryniak503@gmail.com"
__license__ = "MIT License"
__copyright__ = f"Copyright 2021 {__author__}"

